/*
 * 5799-WZQ (C) COPYRIGHT IBM CORPORATION 1986
 * LICENSED MATERIALS - PROPERTY OF IBM
 * REFER TO COPYRIGHT INSTRUCTIONS FORM NUMBER G120-2083
 */
/* $Header:symtab.h 12.0$ */
/* $ACIS:symtab.h 12.0$ */
/* $Source: /ibm/acis/usr/src/ibm/fdb_ca/lib/h/RCS/symtab.h,v $ */

#if !defined(lint) && !defined(LOCORE)  && defined(RCS_HDRS)
static char *rcsidsymtab = "$Header:symtab.h 12.0$";
#endif

/*
 * symtab.h: header file for symtab.c
 */
/* the following prevents duplicate definitions */
#ifndef _symtab
#define _symtab

typedef char *symtab_type;

symtab_type symtab_create ();

int symtab_add();

int symtab_find();

int symtab_scan_reset();

int symtab_scan_next();
 
int symtab_distroy();

#endif _symtab
